﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Imaging;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// Needed for AsStream extension method on IBuffer:
using System.Runtime.InteropServices.WindowsRuntime;

namespace InvertPixels
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
            DecodeUserSelectedFile();
        }

        async Task DecodeUserSelectedFile()
        {
            // Get a JPEG from the user
            FileOpenPicker picker = new FileOpenPicker();
            picker.FileTypeFilter.Add(".jpg");
            picker.FileTypeFilter.Add(".jpeg");
            StorageFile file = await picker.PickSingleFileAsync();
            if (file != null)
            {
                // Get the image data
                using (IRandomAccessStream ras = await file.OpenAsync(FileAccessMode.Read))
                {
                    // Create the decoder
                    BitmapDecoder decoder = await BitmapDecoder.CreateAsync(ras);
                    // Use the decoder
                    UseDecoder(decoder);
                }
            }
        }

        async Task UseDecoder(BitmapDecoder decoder)
        {
            // Force each pixel to be 4 bytes (B, G, R, then A)
            PixelDataProvider provider = await decoder.GetPixelDataAsync(
              BitmapPixelFormat.Bgra8, BitmapAlphaMode.Straight, new BitmapTransform(),
              ExifOrientationMode.RespectExifOrientation,
              ColorManagementMode.ColorManageToSRgb);
            byte[] pixels = provider.DetachPixelData();

            // Invert the colors by subtracting each B, G, and R from 255 (byte.MaxValue)
            for (int i = 0; i < pixels.Length; i += 4)
            {
                pixels[i] = (byte)(byte.MaxValue - pixels[i]);   // B
                pixels[i + 1] = (byte)(byte.MaxValue - pixels[i + 1]); // G
                pixels[i + 2] = (byte)(byte.MaxValue - pixels[i + 2]); // R
                // Leave A alone
            }

            // Create a new WriteableBitmap to contain the edited pixels
            WriteableBitmap bitmap = new WriteableBitmap((int)decoder.OrientedPixelWidth,
                                                         (int)decoder.OrientedPixelHeight);

            // Get a .NET Stream for the bitmap's bytes
            using (Stream stream = bitmap.PixelBuffer.AsStream())
            {
                // Write all pixels using the existing BGRA array 
                await stream.WriteAsync(pixels, 0, pixels.Length);
            }

            // Assign the BitmapImage to the Image element on the page
            image.Source = bitmap;
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }
    }
}
