﻿using System;
using System.IO;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Media.Core;
using Windows.Media.MediaProperties;
using Windows.UI.Xaml.Controls;

namespace MediaElementExample
{
  public sealed partial class MainPage : Page
  {
    // Details about the dynamic audio encoding
    const int SamplesPerSecond = 16000;
    const int ChannelCount = 1; // Mono
    const int BitsPerSample = 8; // Each sample is one byte

    // How frequently data needs to be requested
    // (and how much data is given to each request)
    const int RequestsPerSecond = 10;
    readonly TimeSpan RequestLength = 
      TimeSpan.FromMilliseconds(1000 / RequestsPerSecond);
    const int BitsPerRequest = 
      (SamplesPerSecond / RequestsPerSecond) * BitsPerSample;
    const int BytesPerRequest = BitsPerRequest / 8;

    // Produce 10 seconds of audio
    readonly TimeSpan AudioLength = TimeSpan.FromSeconds(10);

    TimeSpan _elapsedTime = new TimeSpan();
    Random _random = new Random();

    public MainPage()
    {
      this.InitializeComponent();

      // Capture properties about the dynamic audio
      AudioEncodingProperties audioProperties = 
        AudioEncodingProperties.CreatePcm(
        SamplesPerSecond, ChannelCount, BitsPerSample);

      // Create the MediaStreamSource and provide the information it needs
      MediaStreamSource source = new MediaStreamSource(
        new AudioStreamDescriptor(audioProperties));
      source.SampleRequested += MediaStreamSource_SampleRequested;
      source.Duration = AudioLength;

      // These are only relevant for the media transport controls
      source.MusicProperties.Title = "Random Noise";
      source.MusicProperties.Album = "Disturbing Sounds";
      source.MusicProperties.Artist = "Adam Nathan";

      // Auto-play the dynamic audio
      this.mediaElement.SetMediaStreamSource(source);
    }

    async void MediaStreamSource_SampleRequested(MediaStreamSource sender,
      MediaStreamSourceSampleRequestedEventArgs args)
    {
      if (_elapsedTime < sender.Duration)
      {
        MediaStreamSourceSampleRequest request = args.Request;
        MediaStreamSourceSampleRequestDeferral deferral = request.GetDeferral();

        // Generate the random audio data
        byte[] bytes = new byte[BytesPerRequest];
        _random.NextBytes(bytes);

        // Write the data into a .NET stream
        MemoryStream stream = new MemoryStream();
        await stream.WriteAsync(bytes, 0, bytes.Length);

        // Give the data to the request
        request.Sample = MediaStreamSample.CreateFromBuffer(stream.GetWindowsRuntimeBuffer(), _elapsedTime);
        request.Sample.Duration = RequestLength;

        // Track our progress
        _elapsedTime += RequestLength;

        deferral.Complete();
      }
    }
  }
}