﻿using System;
using Windows.Foundation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace CustomPanels
{
  public class UniformGrid : Panel
  {
    int _numRowsAndCols;

    protected override Size MeasureOverride(Size availableSize)
    {
      Size maxChildSize = new Size();

      foreach (UIElement child in this.Children)
      {
        if (child != null)
        {
          // See how big each child wants to be given all our available space
          child.Measure(availableSize);

          // Keep track of the maximum child width and maximum child height
          if (child.DesiredSize.Width > maxChildSize.Width)
          {
            maxChildSize.Width = child.DesiredSize.Width;
          }
          if (child.DesiredSize.Height > maxChildSize.Height)
          {
            maxChildSize.Height = child.DesiredSize.Height;
          }
        }
      }

      // Determine how many rows and columns we need based on the # of children
      _numRowsAndCols = (int)Math.Ceiling(Math.Sqrt(this.Children.Count));

      // Our desired size is the same in each dimension
      Size desiredSize = new Size();
      desiredSize.Width = maxChildSize.Width * _numRowsAndCols;
      desiredSize.Height = maxChildSize.Height * _numRowsAndCols;

      return desiredSize;
    }

    protected override Size ArrangeOverride(Size finalSize)
    {
      double leftOffset = 0;
      double topOffset = 0;
      int childIndex = 0;

      // Give each child the same-sized cell by splitting our final size
      Size finalChildSize = new Size(finalSize.Width / _numRowsAndCols,
                                     finalSize.Height / _numRowsAndCols);

      foreach (UIElement child in this.Children)
      {
        if (child != null)
        {
          // Place the child at (leftOffset, topOffset)
          child.Arrange(new Rect(new Point(leftOffset, topOffset), finalChildSize));

          // Update the offsets for the next child
          childIndex++;
          if (childIndex % _numRowsAndCols == 0)
          {
            // The next child begins the next row
            leftOffset = 0;
            topOffset += finalChildSize.Height;
          }
          else
          {
            // Move to the next column
            leftOffset += finalChildSize.Width;
          }
        }
      }

      // Fill all the space given
      return finalSize;
    }
  }
}